;;; -*- mode: Lisp; Syntax: Common-Lisp; -*-
;;;
;;; Copyright (c) by the authors.
;;;
;;; See LICENCE for details.

(in-package :projectured)

;;;;;;
;;; Document

(def document style/base ()
  ())

(def document style/color (style/base)
  ((red :type number)
   (green :type number)
   (blue :type number)
   (alpha :type number)
   (raw :type t)))

;;;;;;
;;; Construction

(def function make-style/color (red green blue alpha)
  (make-instance 'style/color
                 :red red
                 :green green
                 :blue blue
                 :alpha alpha
                 :raw nil))

;;;;;;
;;; Definition

(def definer color (name (red green blue))
  `(def special-variable ,name (make-style/color ,(/ red 255) ,(/ green 255) ,(/ blue 255) 1.0)))

;;;;;;
;;; Default

(def color *color/default* (0 0 0))

;;;;;;
;;; Pure

(def color *color/black* (0 0 0))
(def color *color/white* (255 255 255))
(def color *color/red* (255 0 0))
(def color *color/green* (0 255 0))
(def color *color/blue* (0 0 255))
(def color *color/yellow* (255 255 0))
(def color *color/purple* (255 0 255))
(def color *color/cyan* (0 255 255))

;;;;;;
;;; Gray

(def color *color/gray0* (0 0 0))
(def color *color/gray15* (15 15 15))
(def color *color/gray31* (31 31 31))
(def color *color/gray47* (47 47 47))
(def color *color/gray63* (63 63 63))
(def color *color/gray79* (79 79 79))
(def color *color/gray95* (95 95 95))
(def color *color/gray111* (111 111 111))
(def color *color/gray127* (127 127 127))
(def color *color/gray143* (143 143 143))
(def color *color/gray159* (159 159 159))
(def color *color/gray175* (175 175 175))
(def color *color/gray191* (191 191 191))
(def color *color/gray207* (207 207 207))
(def color *color/gray223* (223 223 223))
(def color *color/gray239* (239 239 239))
(def color *color/gray255* (255 255 255))

;;;;;;
;;; Solarized
;;;
;;; http://ethanschoonover.com/solarized

(def color *color/solarized/gray* (128 128 128))
(def color *color/solarized/yellow* (181 137 0))
(def color *color/solarized/orange* (203 75 22))
(def color *color/solarized/red* (220 50 47))
(def color *color/solarized/magenta* (211 54 130))
(def color *color/solarized/violet* (108 113 196))
(def color *color/solarized/blue* (38 139 210))
(def color *color/solarized/cyan* (42 161 152))
(def color *color/solarized/green* (133 153 0))
(def color *color/solarized/background/darker* (0 43 54))
(def color *color/solarized/background/dark* (7 54 66))
(def color *color/solarized/background/light* (238 232 213))
(def color *color/solarized/background/lighter* (253 246 227))
(def color *color/solarized/content/darker* (88 110 117))
(def color *color/solarized/content/dark* (101 123 131))
(def color *color/solarized/content/light* (131 148 150))
(def color *color/solarized/content/lighter* (147 161 161))

;;;;;;
;;; Pastel pairs

(def color *color/pastel/turquoise* (26 188 156))
(def color *color/pastel/green-sea* (22 160 133))

(def color *color/pastel/emerland* (46 204 113))
(def color *color/pastel/nephritis* (39 174 96))

(def color *color/pastel/peter-river* (52 152 219))
(def color *color/pastel/belize-hole* (41 128 185))

(def color *color/pastel/amethyst* (155 89 182))
(def color *color/pastel/wisteria* (142 68 173))

(def color *color/pastel/wet-asphalt* (52 73 94))
(def color *color/pastel/midnight-blue* (44 62 80))

(def color *color/pastel/sun-flower* (241 196 15))
(def color *color/pastel/orange* (243 156 18))

(def color *color/pastel/carrot* (230 126 34))
(def color *color/pastel/pumpkin* (211 84 0))

(def color *color/pastel/alizarin* (231 76 60))
(def color *color/pastel/pomegranate* (192 57 43))

(def color *color/pastel/clouds* (236 240 241))
(def color *color/pastel/silver* (189 195 199))

(def color *color/pastel/concrete* (149 165 166))
(def color *color/pastel/asbestos* (127 140 141))

;;;;;;
;;; Named
;;;
;;; http://en.wikipedia.org/wiki/List_of_colors_(compact)

(def color *color/aero* (124 185 232))
(def color *color/aero-blue* (201 255 229))
(def color *color/african-violet* (178 132 190))
(def color *color/air-force-blue-raf* (93 138 168))
(def color *color/air-force-blue-usaf* (0 48 143))
(def color *color/air-superiority-blue* (114 160 193))
(def color *color/alabama-crimson* (163 38 56))
(def color *color/alice-blue* (240 248 255))
(def color *color/alizarin-crimson* (227 38 54))
(def color *color/alloy-orange* (196 98 16))
(def color *color/almond* (239 222 205))
(def color *color/amaranth* (229 43 80))
(def color *color/amazon* (59 122 87))
(def color *color/amber* (255 191 0))
(def color *color/ece-amber* (255 126 0))
(def color *color/american-rose* (255 3 62))
(def color *color/android-green* (164 198 57))
(def color *color/anti-flash-white* (242 243 244))
(def color *color/antique-brass* (205 149 117))
(def color *color/antique-bronze* (102 93 30))
(def color *color/antique-fuchsia* (145 92 131))
(def color *color/antique-ruby* (132 27 45))
(def color *color/antique-white* (250 235 215))
(def color *color/ao* (0 128 0))
(def color *color/apple-green* (141 182 0))
(def color *color/apricot* (251 206 177))
(def color *color/aquamarine* (127 255 212))
(def color *color/army-green* (75 83 32))
(def color *color/arsenic* (59 68 75))
(def color *color/arylide-yellow* (233 214 107))
(def color *color/ash-grey* (178 190 181))
(def color *color/asparagus* (135 169 107))
(def color *color/atomic-tangerine* (255 153 102))
(def color *color/auburn* (165 42 42))
(def color *color/aureolin* (253 238 0))
(def color *color/aurometalsaurus* (110 127 128))
(def color *color/avocado* (86 130 3))
(def color *color/azure* (0 127 255))
(def color *color/azure-mist* (240 255 255))
(def color *color/baby-blue* (137 207 240))
(def color *color/baby-blue-eyes* (161 202 241))
(def color *color/baby-pink* (244 194 194))
(def color *color/baby-powder* (254 254 250))
(def color *color/baker-miller-pink* (255 145 175))
(def color *color/ball-blue* (33 171 205))
(def color *color/banana-mania* (250 231 181))
(def color *color/banana-yellow* (255 225 53))
(def color *color/barbie-pink* (224 33 138))
(def color *color/barn-red* (124 10 2))
(def color *color/battleship-grey* (132 132 130))
(def color *color/bazaar* (152 119 123))
(def color *color/bdazzled-blue* (46 88 148))
(def color *color/beau-blue* (188 212 230))
(def color *color/beaver* (159 129 112))
(def color *color/beige* (245 245 220))
(def color *color/big-dip-o’ruby* (156 37 66))
(def color *color/bisque* (255 228 196))
(def color *color/bistre* (61 43 31))
(def color *color/bistre-brown* (150 113 23))
(def color *color/bitter-lemon* (202 224 13))
(def color *color/bitter-lime* (100 140 17))
(def color *color/bittersweet* (254 111 94))
(def color *color/bittersweet-shimmer* (191 79 81))
(def color *color/black-bean* (61 12 2))
(def color *color/black-leather-jacket* (37 53 41))
(def color *color/black-olive* (59 60 54))
(def color *color/blanched-almond* (255 235 205))
(def color *color/blast-off-bronze* (165 113 100))
(def color *color/bleu-de-france* (49 140 231))
(def color *color/blizzard-blue* (172 229 238))
(def color *color/blond* (250 240 190))
(def color *color/blue-crayola* (31 117 254))
(def color *color/blue-munsell* (0 147 175))
(def color *color/blue-ncs* (0 135 189))
(def color *color/blue-pigment* (51 51 153))
(def color *color/blue-ryb* (2 71 254))
(def color *color/blue-bell* (162 162 208))
(def color *color/blue-gray* (102 153 204))
(def color *color/blue-green* (13 152 186))
(def color *color/blue-sapphire* (18 97 128))
(def color *color/blue-violet* (138 43 226))
(def color *color/blueberry* (79 134 247))
(def color *color/bluebonnet* (28 28 240))
(def color *color/blush* (222 93 131))
(def color *color/bole* (121 68 59))
(def color *color/bondi-blue* (0 149 182))
(def color *color/bone* (227 218 201))
(def color *color/boston-university-red* (204 0 0))
(def color *color/bottle-green* (0 106 78))
(def color *color/boysenberry* (135 50 96))
(def color *color/brandeis-blue* (0 112 255))
(def color *color/brass* (181 166 66))
(def color *color/brick-red* (203 65 84))
(def color *color/bright-cerulean* (29 172 214))
(def color *color/bright-lavender* (191 148 228))
(def color *color/bright-maroon* (195 33 72))
(def color *color/bright-pink* (255 0 127))
(def color *color/bright-turquoise* (8 232 222))
(def color *color/bright-ube* (209 159 232))
(def color *color/brilliant-lavender* (244 187 255))
(def color *color/brilliant-rose* (255 85 163))
(def color *color/brink-pink* (251 96 127))
(def color *color/british-racing-green* (0 66 37))
(def color *color/bronze* (205 127 50))
(def color *color/bronze-yellow* (115 112 0))
(def color *color/brown-traditional* (150 75 0))
(def color *color/brown-web* (165 42 42))
(def color *color/brown-nose* (102 68 35))
(def color *color/brunswick-green* (27 77 62))
(def color *color/bubble-gum* (255 193 204))
(def color *color/bubbles* (231 254 255))
(def color *color/buff* (240 220 130))
(def color *color/bulgarian-rose* (72 6 7))
(def color *color/burgundy* (128 0 32))
(def color *color/burlywood* (222 184 135))
(def color *color/burnt-orange* (204 85 0))
(def color *color/burnt-sienna* (233 116 81))
(def color *color/burnt-umber* (138 51 36))
(def color *color/byzantine* (189 51 164))
(def color *color/byzantium* (112 41 99))
(def color *color/cadet* (83 104 114))
(def color *color/cadet-blue* (95 158 160))
(def color *color/cadet-grey* (145 163 176))
(def color *color/cadmium-green* (0 107 60))
(def color *color/cadmium-orange* (237 135 45))
(def color *color/cadmium-red* (227 0 34))
(def color *color/cadmium-yellow* (255 246 0))
(def color *color/café-au-lait* (166 123 91))
(def color *color/café-noir* (75 54 33))
(def color *color/cal-poly-green* (30 77 43))
(def color *color/cambridge-blue* (163 193 173))
(def color *color/camel* (193 154 107))
(def color *color/cameo-pink* (239 187 204))
(def color *color/camouflage-green* (120 134 107))
(def color *color/canary-yellow* (255 239 0))
(def color *color/candy-apple-red* (255 8 0))
(def color *color/candy-pink* (228 113 122))
(def color *color/capri* (0 191 255))
(def color *color/caput-mortuum* (89 39 32))
(def color *color/cardinal* (196 30 58))
(def color *color/caribbean-green* (0 204 153))
(def color *color/carmine* (150 0 24))
(def color *color/carmine-m&p* (215 0 64))
(def color *color/carmine-pink* (235 76 66))
(def color *color/carmine-red* (255 0 56))
(def color *color/carnation-pink* (255 166 201))
(def color *color/carnelian* (179 27 27))
(def color *color/carolina-blue* (153 186 221))
(def color *color/carrot-orange* (237 145 33))
(def color *color/castleton-green* (0 86 63))
(def color *color/catalina-blue* (6 42 120))
(def color *color/catawba* (112 54 66))
(def color *color/cedar-chest* (201 90 73))
(def color *color/ceil* (146 161 207))
(def color *color/celadon* (172 225 175))
(def color *color/celadon-blue* (0 123 167))
(def color *color/celadon-green* (47 132 124))
(def color *color/celeste-colour* (178 255 255))
(def color *color/celestial-blue* (73 151 208))
(def color *color/cerise* (222 49 99))
(def color *color/cerise-pink* (236 59 131))
(def color *color/cerulean* (0 123 167))
(def color *color/cerulean-blue* (42 82 190))
(def color *color/cerulean-frost* (109 155 195))
(def color *color/cg-blue* (0 122 165))
(def color *color/cg-red* (224 60 49))
(def color *color/chamoisee* (160 120 90))
(def color *color/champagne* (247 231 206))
(def color *color/charcoal* (54 69 79))
(def color *color/charleston-green* (35 43 43))
(def color *color/charm-pink* (230 143 172))
(def color *color/chartreuse-traditional* (223 255 0))
(def color *color/chartreuse-web* (127 255 0))
(def color *color/cherry* (222 49 99))
(def color *color/cherry-blossom-pink* (255 183 197))
(def color *color/chestnut* (144 69 53))
(def color *color/china-pink* (222 111 161))
(def color *color/china-rose* (168 81 110))
(def color *color/chinese-red* (170 56 30))
(def color *color/chinese-violet* (133 96 136))
(def color *color/chocolate-traditional* (123 63 0))
(def color *color/chocolate-web* (210 105 30))
(def color *color/chrome-yellow* (255 167 0))
(def color *color/cinereous* (152 129 123))
(def color *color/cinnabar* (227 66 52))
(def color *color/cinnamon* (210 105 30))
(def color *color/citrine* (228 208 10))
(def color *color/citron* (158 169 31))
(def color *color/claret* (127 23 52))
(def color *color/classic-rose* (251 204 231))
(def color *color/cobalt* (0 71 171))
(def color *color/cocoa-brown* (210 105 30))
(def color *color/coconut* (150 90 62))
(def color *color/coffee* (111 78 55))
(def color *color/columbia-blue* (155 221 255))
(def color *color/congo-pink* (248 131 121))
(def color *color/cool-black* (0 46 99))
(def color *color/cool-grey* (140 146 172))
(def color *color/copper* (184 115 51))
(def color *color/copper-crayola* (218 138 103))
(def color *color/copper-penny* (173 111 105))
(def color *color/copper-red* (203 109 81))
(def color *color/coquelicot* (255 56 0))
(def color *color/coral* (255 127 80))
(def color *color/coral-pink* (248 131 121))
(def color *color/coral-red* (255 64 64))
(def color *color/cordovan* (137 63 69))
(def color *color/corn* (251 236 93))
(def color *color/cornell-red* (179 27 27))
(def color *color/cornflower-blue* (100 149 237))
(def color *color/cornsilk* (255 248 220))
(def color *color/cosmic-latte* (255 248 231))
(def color *color/cotton-candy* (255 188 217))
(def color *color/cream* (255 253 208))
(def color *color/crimson* (220 20 60))
(def color *color/crimson-glory* (190 0 50))
(def color *color/cyan-process* (0 183 235))
(def color *color/cyber-grape* (88 66 124))
(def color *color/daffodil* (255 255 49))
(def color *color/dandelion* (240 225 48))
(def color *color/dark-blue* (0 0 139))
(def color *color/dark-blue-gray* (102 102 153))
(def color *color/dark-brown* (101 67 33))
(def color *color/dark-byzantium* (93 57 84))
(def color *color/dark-candy-apple-red* (164 0 0))
(def color *color/dark-cerulean* (8 69 126))
(def color *color/dark-chestnut* (152 105 96))
(def color *color/dark-coral* (205 91 69))
(def color *color/dark-cyan* (0 139 139))
(def color *color/dark-electric-blue* (83 104 120))
(def color *color/dark-goldenrod* (184 134 11))
(def color *color/dark-gray* (169 169 169))
(def color *color/dark-green* (1 50 32))
(def color *color/dark-imperial-blue* (0 65 106))
(def color *color/dark-jungle-green* (26 36 33))
(def color *color/dark-khaki* (189 183 107))
(def color *color/dark-lava* (72 60 50))
(def color *color/dark-lavender* (115 79 150))
(def color *color/dark-magenta* (139 0 139))
(def color *color/dark-midnight-blue* (0 51 102))
(def color *color/dark-moss-green* (74 93 35))
(def color *color/dark-olive-green* (85 107 47))
(def color *color/dark-orange* (255 140 0))
(def color *color/dark-orchid* (153 50 204))
(def color *color/dark-pastel-blue* (119 158 203))
(def color *color/dark-pastel-green* (3 192 60))
(def color *color/dark-pastel-purple* (150 111 214))
(def color *color/dark-pastel-red* (194 59 34))
(def color *color/dark-pink* (231 84 128))
(def color *color/dark-powder-blue* (0 51 153))
(def color *color/dark-raspberry* (135 38 87))
(def color *color/dark-red* (139 0 0))
(def color *color/dark-salmon* (233 150 122))
(def color *color/dark-scarlet* (86 3 25))
(def color *color/dark-sea-green* (143 188 143))
(def color *color/dark-sienna* (60 20 20))
(def color *color/dark-sky-blue* (140 190 214))
(def color *color/dark-slate-blue* (72 61 139))
(def color *color/dark-slate-gray* (47 79 79))
(def color *color/dark-spring-green* (23 114 69))
(def color *color/dark-tan* (145 129 81))
(def color *color/dark-tangerine* (255 168 18))
(def color *color/dark-taupe* (72 60 50))
(def color *color/dark-terra-cotta* (204 78 92))
(def color *color/dark-turquoise* (0 206 209))
(def color *color/dark-vanilla* (209 190 168))
(def color *color/dark-violet* (148 0 211))
(def color *color/dark-yellow* (155 135 12))
(def color *color/dartmouth-green* (0 112 60))
(def color *color/davys-grey* (85 85 85))
(def color *color/debian-red* (215 10 83))
(def color *color/deep-carmine* (169 32 62))
(def color *color/deep-carmine-pink* (239 48 56))
(def color *color/deep-carrot-orange* (233 105 44))
(def color *color/deep-cerise* (218 50 135))
(def color *color/deep-champagne* (250 214 165))
(def color *color/deep-chestnut* (185 78 72))
(def color *color/deep-coffee* (112 66 65))
(def color *color/deep-fuchsia* (193 84 193))
(def color *color/deep-jungle-green* (0 75 73))
(def color *color/deep-lemon* (245 199 76))
(def color *color/deep-lilac* (153 85 187))
(def color *color/deep-magenta* (204 0 204))
(def color *color/deep-mauve* (212 115 212))
(def color *color/deep-moss-green* (53 94 59))
(def color *color/deep-peach* (255 203 164))
(def color *color/deep-pink* (255 20 147))
(def color *color/deep-ruby* (132 63 91))
(def color *color/deep-saffron* (255 153 51))
(def color *color/deep-sky-blue* (0 191 255))
(def color *color/deep-space-sparkle* (74 100 108))
(def color *color/deep-taupe* (126 94 96))
(def color *color/deep-tuscan-red* (102 66 77))
(def color *color/deer* (186 135 89))
(def color *color/denim* (21 96 189))
(def color *color/desert* (193 154 107))
(def color *color/desert-sand* (237 201 175))
(def color *color/diamond* (185 242 255))
(def color *color/dim-gray* (105 105 105))
(def color *color/dirt* (155 118 83))
(def color *color/dodger-blue* (30 144 255))
(def color *color/dogwood-rose* (215 24 104))
(def color *color/dollar-bill* (133 187 101))
(def color *color/drab* (150 113 23))
(def color *color/duke-blue* (0 0 156))
(def color *color/dust-storm* (229 204 201))
(def color *color/earth-yellow* (225 169 95))
(def color *color/ebony* (85 93 80))
(def color *color/ecru* (194 178 128))
(def color *color/eggplant* (97 64 81))
(def color *color/eggshell* (240 234 214))
(def color *color/egyptian-blue* (16 52 166))
(def color *color/electric-blue* (125 249 255))
(def color *color/electric-crimson* (255 0 63))
(def color *color/electric-cyan* (0 255 255))
(def color *color/electric-indigo* (111 0 255))
(def color *color/electric-lavender* (244 187 255))
(def color *color/electric-purple* (191 0 255))
(def color *color/electric-ultramarine* (63 0 255))
(def color *color/electric-violet* (143 0 255))
(def color *color/electric-yellow* (255 255 51))
(def color *color/emerald* (80 200 120))
(def color *color/english-green* (27 77 62))
(def color *color/english-lavender* (180 131 149))
(def color *color/english-red* (171 78 82))
(def color *color/english-violet* (86 60 92))
(def color *color/eton-blue* (150 200 162))
(def color *color/eucalyptus* (68 215 168))
(def color *color/fallow* (193 154 107))
(def color *color/falu-red* (128 24 24))
(def color *color/fandango* (181 51 137))
(def color *color/fandango-pink* (222 82 133))
(def color *color/fashion-fuchsia* (244 0 161))
(def color *color/fawn* (229 170 112))
(def color *color/feldgrau* (77 93 83))
(def color *color/feldspar* (224 181 130))
(def color *color/fern-green* (79 121 66))
(def color *color/ferrari-red* (255 40 0))
(def color *color/field-drab* (108 84 30))
(def color *color/firebrick* (178 34 34))
(def color *color/fire-engine-red* (206 32 41))
(def color *color/flame* (226 88 34))
(def color *color/flamingo-pink* (252 142 172))
(def color *color/flattery* (102 68 35))
(def color *color/flavescent* (247 233 142))
(def color *color/flax* (238 220 130))
(def color *color/floral-white* (255 250 240))
(def color *color/fluorescent-orange* (255 191 0))
(def color *color/fluorescent-pink* (255 20 147))
(def color *color/fluorescent-yellow* (204 255 0))
(def color *color/folly* (255 0 79))
(def color *color/forest-green-traditional* (1 68 33))
(def color *color/forest-green-web* (34 139 34))
(def color *color/french-beige* (166 123 91))
(def color *color/french-bistre* (133 109 77))
(def color *color/french-blue* (0 114 187))
(def color *color/french-lilac* (134 96 142))
(def color *color/french-lime* (158 253 56))
(def color *color/french-mauve* (212 115 212))
(def color *color/french-raspberry* (199 44 72))
(def color *color/french-rose* (246 74 138))
(def color *color/french-sky-blue* (119 181 254))
(def color *color/french-wine* (172 30 68))
(def color *color/fresh-air* (166 231 255))
(def color *color/fuchsia-crayola* (193 84 193))
(def color *color/fuchsia-pink* (255 119 255))
(def color *color/fuchsia-rose* (199 67 117))
(def color *color/fulvous* (228 132 0))
(def color *color/fuzzy-wuzzy* (204 102 102))
(def color *color/gainsboro* (220 220 220))
(def color *color/gamboge* (228 155 15))
(def color *color/ghost-white* (248 248 255))
(def color *color/giants-orange* (254 90 29))
(def color *color/ginger* (176 101 0))
(def color *color/glaucous* (96 130 182))
(def color *color/glitter* (230 232 250))
(def color *color/go-green* (0 171 102))
(def color *color/gold-metallic* (212 175 55))
(def color *color/html_gold* (255 215 0))
(def color *color/gold-fusion* (133 117 78))
(def color *color/golden-brown* (153 101 21))
(def color *color/golden-poppy* (252 194 0))
(def color *color/golden-yellow* (255 223 0))
(def color *color/goldenrod* (218 165 32))
(def color *color/granny-smith-apple* (168 228 160))
(def color *color/grape* (106 45 168))
(def color *color/html_grey* (128 128 128))
(def color *color/gray-html/css-gray* (128 128 128))
(def color *color/gray-x11-gray* (190 190 190))
(def color *color/gray-asparagus* (70 89 69))
(def color *color/gray-blue* (140 146 172))
(def color *color/html_lime* (0 255 0))
(def color *color/green-crayola* (28 172 120))
(def color *color/html_green* (0 128 0))
(def color *color/green-munsell* (0 168 119))
(def color *color/green-ncs* (0 159 107))
(def color *color/green-pigment* (0 165 80))
(def color *color/green-ryb* (102 176 50))
(def color *color/green-yellow* (173 255 47))
(def color *color/grullo* (169 154 134))
(def color *color/guppie-green* (0 255 127))
(def color *color/halayà-úbe* (102 55 84))
(def color *color/han-blue* (68 108 207))
(def color *color/han-purple* (82 24 250))
(def color *color/hansa-yellow* (233 214 107))
(def color *color/harlequin* (63 255 0))
(def color *color/harvard-crimson* (201 0 22))
(def color *color/harvest-gold* (218 145 0))
(def color *color/heart-gold* (128 128 0))
(def color *color/heliotrope* (223 115 255))
(def color *color/hollywood-cerise* (244 0 161))
(def color *color/honeydew* (240 255 240))
(def color *color/honolulu-blue* (0 109 176))
(def color *color/hookers-green* (73 121 107))
(def color *color/hot-magenta* (255 29 206))
(def color *color/hot-pink* (255 105 180))
(def color *color/hunter-green* (53 94 59))
(def color *color/iceberg* (113 166 210))
(def color *color/icterine* (252 247 94))
(def color *color/illuminating-emerald* (49 145 119))
(def color *color/imperial* (96 47 107))
(def color *color/imperial-blue* (0 35 149))
(def color *color/imperial-purple* (102 2 60))
(def color *color/imperial-red* (237 41 57))
(def color *color/inchworm* (178 236 93))
(def color *color/india-green* (19 136 8))
(def color *color/indian-red* (205 92 92))
(def color *color/indian-yellow* (227 168 87))
(def color *color/indigo* (111 0 255))
(def color *color/indigo-dye* (0 65 106))
(def color *color/indigo-web* (75 0 130))
(def color *color/international-klein-blue* (0 47 167))
(def color *color/international-orange-aerospace* (255 79 0))
(def color *color/international-orange-engineering* (186 22 12))
(def color *color/international-orange-golden-gate-bridge* (192 54 44))
(def color *color/iris* (90 79 207))
(def color *color/irresistible* (179 68 108))
(def color *color/isabelline* (244 240 236))
(def color *color/islamic-green* (0 144 0))
(def color *color/italian-sky-blue* (178 255 255))
(def color *color/jade* (0 168 107))
(def color *color/japanese-indigo* (38 67 72))
(def color *color/japanese-violet* (91 50 86))
(def color *color/jasmine* (248 222 126))
(def color *color/jasper* (215 59 62))
(def color *color/jazzberry-jam* (165 11 94))
(def color *color/jelly-bean* (218 97 78))
(def color *color/jet* (52 52 52))
(def color *color/jonquil* (244 202 22))
(def color *color/june-bud* (189 218 87))
(def color *color/jungle-green* (41 171 135))
(def color *color/kelly-green* (76 187 23))
(def color *color/kenyan-copper* (124 28 5))
(def color *color/keppel* (58 176 158))
(def color *color/khaki-html/css-khaki* (195 176 145))
(def color *color/khaki-x11-light-khaki* (240 230 140))
(def color *color/kobe* (136 45 23))
(def color *color/kobi* (231 159 196))
(def color *color/ku-crimson* (232 0 13))
(def color *color/la-salle-green* (8 120 48))
(def color *color/languid-lavender* (214 202 221))
(def color *color/lapis-lazuli* (38 97 156))
(def color *color/laser-lemon* (255 255 102))
(def color *color/laurel-green* (169 186 157))
(def color *color/lava* (207 16 32))
(def color *color/lavender-floral* (181 126 220))
(def color *color/lavender-web* (230 230 250))
(def color *color/lavender-blush* (255 240 245))
(def color *color/lavender-gray* (196 195 208))
(def color *color/lavender-indigo* (148 87 235))
(def color *color/lavender-magenta* (238 130 238))
(def color *color/lavender-mist* (230 230 250))
(def color *color/lavender-pink* (251 174 210))
(def color *color/lavender-purple* (150 123 182))
(def color *color/lavender-rose* (251 160 227))
(def color *color/lawn-green* (124 252 0))
(def color *color/lemon* (255 247 0))
(def color *color/lemon-chiffon* (255 250 205))
(def color *color/lemon-curry* (204 160 29))
(def color *color/lemon-glacier* (253 255 0))
(def color *color/lemon-lime* (227 255 0))
(def color *color/lemon-meringue* (246 234 190))
(def color *color/lemon-yellow* (255 244 79))
(def color *color/licorice* (26 17 16))
(def color *color/light-apricot* (253 213 177))
(def color *color/light-blue* (173 216 230))
(def color *color/light-brown* (181 101 29))
(def color *color/light-carmine-pink* (230 103 113))
(def color *color/light-coral* (240 128 128))
(def color *color/light-cornflower-blue* (147 204 234))
(def color *color/light-crimson* (245 105 145))
(def color *color/light-cyan* (224 255 255))
(def color *color/light-fuchsia-pink* (249 132 239))
(def color *color/light-goldenrod-yellow* (250 250 210))
(def color *color/light-gray* (211 211 211))
(def color *color/light-green* (144 238 144))
(def color *color/light-khaki* (240 230 140))
(def color *color/light-medium-orchid* (211 155 203))
(def color *color/light-moss-green* (173 200 173))
(def color *color/light-orchid* (230 168 215))
(def color *color/light-pastel-purple* (177 156 217))
(def color *color/light-pink* (255 182 193))
(def color *color/light-red-ochre* (233 116 81))
(def color *color/light-salmon* (255 160 122))
(def color *color/light-salmon-pink* (255 153 153))
(def color *color/light-sea-green* (32 178 170))
(def color *color/light-sky-blue* (135 206 250))
(def color *color/light-slate-gray* (119 136 153))
(def color *color/light-steel-blue* (176 196 222))
(def color *color/light-taupe* (179 139 109))
(def color *color/light-thulian-pink* (230 143 172))
(def color *color/light-yellow* (255 255 224))
(def color *color/lilac* (200 162 200))
(def color *color/lime-color-wheel* (191 255 0))
(def color *color/lime-green* (50 205 50))
(def color *color/limerick* (157 194 9))
(def color *color/lincoln-green* (25 89 5))
(def color *color/linen* (250 240 230))
(def color *color/lion* (193 154 107))
(def color *color/little-boy-blue* (108 160 220))
(def color *color/liver* (83 75 79))
(def color *color/lumber* (255 228 205))
(def color *color/lust* (230 32 32))
(def color *color/magenta-crayola* (255 85 163))
(def color *color/magenta-dye* (202 31 123))
(def color *color/magenta-pantone* (208 65 126))
(def color *color/magenta-process* (255 0 144))
(def color *color/magic-mint* (170 240 209))
(def color *color/magnolia* (248 244 255))
(def color *color/mahogany* (192 64 0))
(def color *color/maize* (251 236 93))
(def color *color/majorelle-blue* (96 80 220))
(def color *color/malachite* (11 218 81))
(def color *color/manatee* (151 154 170))
(def color *color/mango-tango* (255 130 67))
(def color *color/mantis* (116 195 101))
(def color *color/mardi-gras* (136 0 137))
(def color *color/maroon-crayola* (195 33 72))
(def color *color/html_maroon* (128 0 0))
(def color *color/maroon-x11* (176 48 96))
(def color *color/mauve* (224 176 255))
(def color *color/mauve-taupe* (145 95 109))
(def color *color/mauvelous* (239 152 170))
(def color *color/maya-blue* (115 194 251))
(def color *color/meat-brown* (229 183 59))
(def color *color/medium-aquamarine* (102 221 170))
(def color *color/medium-blue* (0 0 205))
(def color *color/medium-candy-apple-red* (226 6 44))
(def color *color/medium-carmine* (175 64 53))
(def color *color/medium-champagne* (243 229 171))
(def color *color/medium-electric-blue* (3 80 150))
(def color *color/medium-jungle-green* (28 53 45))
(def color *color/medium-orchid* (186 85 211))
(def color *color/medium-persian-blue* (0 103 165))
(def color *color/medium-purple* (147 112 219))
(def color *color/medium-red-violet* (187 51 133))
(def color *color/medium-ruby* (170 64 105))
(def color *color/medium-sea-green* (60 179 113))
(def color *color/medium-slate-blue* (123 104 238))
(def color *color/medium-spring-bud* (201 220 135))
(def color *color/medium-spring-green* (0 250 154))
(def color *color/medium-sky-blue* (128 218 235))
(def color *color/medium-taupe* (103 76 71))
(def color *color/medium-turquoise* (72 209 204))
(def color *color/medium-tuscan-red* (121 68 59))
(def color *color/medium-vermilion* (217 96 59))
(def color *color/medium-violet-red* (199 21 133))
(def color *color/mellow-apricot* (248 184 120))
(def color *color/mellow-yellow* (248 222 126))
(def color *color/melon* (253 188 180))
(def color *color/metallic-seaweed* (8 126 140))
(def color *color/metallic-sunburst* (156 124 56))
(def color *color/mexican-pink* (228 0 124))
(def color *color/midori* (227 249 136))
(def color *color/mikado-yellow* (255 196 12))
(def color *color/mint* (62 180 137))
(def color *color/mint-cream* (245 255 250))
(def color *color/mint-green* (152 255 152))
(def color *color/misty-rose* (255 228 225))
(def color *color/moccasin* (250 235 215))
(def color *color/mode-beige* (150 113 23))
(def color *color/moonstone-blue* (115 169 194))
(def color *color/mordant-red-19* (174 12 0))
(def color *color/moss-green* (138 154 91))
(def color *color/mountain-meadow* (48 186 143))
(def color *color/mountbatten-pink* (153 122 141))
(def color *color/mulberry* (197 75 140))
(def color *color/mustard* (255 219 88))
(def color *color/myrtle-green* (49 120 115))
(def color *color/nadeshiko-pink* (246 173 198))
(def color *color/napier-green* (42 128 0))
(def color *color/naples-yellow* (250 218 94))
(def color *color/navajo-white* (255 222 173))
(def color *color/navy-blue* (0 0 128))
(def color *color/navy-purple* (148 87 235))
(def color *color/neon-carrot* (255 163 67))
(def color *color/neon-fuchsia* (254 65 100))
(def color *color/neon-green* (57 255 20))
(def color *color/new-car* (33 79 198))
(def color *color/new-york-pink* (215 131 127))
(def color *color/non-photo-blue* (164 221 237))
(def color *color/nyanza* (233 255 219))
(def color *color/ocean-boat-blue* (0 119 190))
(def color *color/ochre* (204 119 34))
(def color *color/html_green* (0 128 0))
(def color *color/old-burgundy* (67 48 46))
(def color *color/old-gold* (207 181 59))
(def color *color/old-lace* (253 245 230))
(def color *color/old-lavender* (121 104 120))
(def color *color/old-mauve* (103 49 71))
(def color *color/old-moss-green* (134 126 54))
(def color *color/old-rose* (192 128 129))
(def color *color/old-silver* (132 132 130))
(def color *color/olive* (128 128 0))
(def color *color/olive-drab-web-olive-drab* (107 142 35))
(def color *color/olive-drab* (60 52 31))
(def color *color/olivine* (154 185 115))
(def color *color/onyx* (53 56 57))
(def color *color/opera-mauve* (183 132 167))
(def color *color/orange-color-wheel* (255 127 0))
(def color *color/orange-crayola* (255 117 56))
(def color *color/orange-pantone* (255 88 0))
(def color *color/orange-ryb* (251 153 2))
(def color *color/orange-web-color* (255 165 0))
(def color *color/orange-peel* (255 159 0))
(def color *color/orange-red* (255 69 0))
(def color *color/orchid* (218 112 214))
(def color *color/orchid-pink* (242 189 205))
(def color *color/orioles-orange* (251 79 20))
(def color *color/otter-brown* (101 67 33))
(def color *color/outer-space* (65 74 76))
(def color *color/outrageous-orange* (255 110 74))
(def color *color/oxford-blue* (0 33 71))
(def color *color/ou-crimson-red* (153 0 0))
(def color *color/pakistan-green* (0 102 0))
(def color *color/palatinate-blue* (39 59 226))
(def color *color/palatinate-purple* (104 40 96))
(def color *color/pale-aqua* (188 212 230))
(def color *color/pale-blue* (175 238 238))
(def color *color/pale-brown* (152 118 84))
(def color *color/pale-carmine* (175 64 53))
(def color *color/pale-cerulean* (155 196 226))
(def color *color/pale-chestnut* (221 173 175))
(def color *color/pale-copper* (218 138 103))
(def color *color/pale-cornflower-blue* (171 205 239))
(def color *color/pale-gold* (230 190 138))
(def color *color/pale-goldenrod* (238 232 170))
(def color *color/pale-green* (152 251 152))
(def color *color/pale-lavender* (220 208 255))
(def color *color/pale-magenta* (249 132 229))
(def color *color/pale-pink* (250 218 221))
(def color *color/pale-red-violet* (219 112 147))
(def color *color/pale-robin-egg-blue* (150 222 209))
(def color *color/pale-silver* (201 192 187))
(def color *color/pale-spring-bud* (236 235 189))
(def color *color/pale-taupe* (188 152 126))
(def color *color/pale-turquoise* (175 238 238))
(def color *color/pale-violet-red* (219 112 147))
(def color *color/pansy-purple* (120 24 74))
(def color *color/papaya-whip* (255 239 213))
(def color *color/paris-green* (80 200 120))
(def color *color/pastel-blue* (174 198 207))
(def color *color/pastel-brown* (130 105 83))
(def color *color/pastel-gray* (207 207 196))
(def color *color/pastel-green* (119 221 119))
(def color *color/pastel-magenta* (244 154 194))
(def color *color/pastel-orange* (255 179 71))
(def color *color/pastel-pink* (222 165 164))
(def color *color/pastel-purple* (179 158 181))
(def color *color/pastel-red* (255 105 97))
(def color *color/pastel-violet* (203 153 201))
(def color *color/pastel-yellow* (253 253 150))
(def color *color/patriarch* (128 0 128))
(def color *color/paynes-grey* (83 104 120))
(def color *color/peach* (255 229 180))
(def color *color/peach-crayola* (255 203 164))
(def color *color/peach-puff* (255 218 185))
(def color *color/peach-yellow* (250 223 173))
(def color *color/pear* (209 226 49))
(def color *color/pearl* (234 224 200))
(def color *color/pearl-aqua* (136 216 192))
(def color *color/pearly-purple* (183 104 162))
(def color *color/peridot* (230 226 0))
(def color *color/persian-blue* (28 57 187))
(def color *color/persian-green* (0 166 147))
(def color *color/persian-indigo* (50 18 122))
(def color *color/persian-orange* (217 144 88))
(def color *color/persian-pink* (247 127 190))
(def color *color/persian-plum* (112 28 28))
(def color *color/persian-rose* (254 40 162))
(def color *color/persimmon* (236 88 0))
(def color *color/peru* (205 133 63))
(def color *color/phlox* (223 0 255))
(def color *color/phthalo-blue* (0 15 137))
(def color *color/phthalo-green* (18 53 36))
(def color *color/pictorial-carmine* (195 11 78))
(def color *color/piggy-pink* (253 221 230))
(def color *color/pine-green* (1 121 111))
(def color *color/pink* (255 192 203))
(def color *color/pink-lace* (255 221 244))
(def color *color/pink-orange* (255 153 102))
(def color *color/pink-pearl* (231 172 207))
(def color *color/pink-sherbet* (247 143 167))
(def color *color/pistachio* (147 197 114))
(def color *color/plum-traditional* (142 69 133))
(def color *color/pomp-and-power* (134 96 142))
(def color *color/portland-orange* (255 90 54))
(def color *color/princeton-orange* (255 143 0))
(def color *color/prune* (112 28 28))
(def color *color/prussian-blue* (0 49 83))
(def color *color/psychedelic-purple* (223 0 255))
(def color *color/puce* (204 136 153))
(def color *color/pumpkin* (255 117 24))
(def color *color/html-purple* (128 0 128))
(def color *color/purple-munsell* (159 0 197))
(def color *color/purple-x11* (160 32 240))
(def color *color/purple-heart* (105 53 156))
(def color *color/purple-mountain-majesty* (150 120 182))
(def color *color/purple-pizzazz* (254 78 218))
(def color *color/purple-taupe* (80 64 77))
(def color *color/quartz* (81 72 79))
(def color *color/queen-blue* (67 107 149))
(def color *color/queen-pink* (232 204 215))
(def color *color/rackley* (93 138 168))
(def color *color/radical-red* (255 53 94))
(def color *color/rajah* (251 174 96))
(def color *color/raspberry* (227 11 92))
(def color *color/raspberry-glace* (145 95 109))
(def color *color/raspberry-pink* (226 80 152))
(def color *color/raspberry-rose* (179 68 108))
(def color *color/raw-umber* (130 102 68))
(def color *color/razzle-dazzle-rose* (255 51 204))
(def color *color/razzmatazz* (227 37 107))
(def color *color/razzmic-berry* (141 78 133))
(def color *color/red-crayola* (238 32 77))
(def color *color/red-munsell* (242 0 60))
(def color *color/red-ncs* (196 2 51))
(def color *color/red-pantone* (237 41 57))
(def color *color/red-pigment* (237 28 36))
(def color *color/red-brown* (165 42 42))
(def color *color/red-devil* (134 1 17))
(def color *color/red-orange* (255 83 73))
(def color *color/red-violet* (199 21 133))
(def color *color/redwood* (164 90 82))
(def color *color/regalia* (82 45 128))
(def color *color/resolution-blue* (0 35 135))
(def color *color/rhythm* (119 118 150))
(def color *color/rich-black* (0 64 64))
(def color *color/rich-brilliant-lavender* (241 167 254))
(def color *color/rich-carmine* (215 0 64))
(def color *color/rich-electric-blue* (8 146 208))
(def color *color/rich-lavender* (167 107 207))
(def color *color/rich-lilac* (182 102 210))
(def color *color/rich-maroon* (176 48 96))
(def color *color/rifle-green* (68 76 56))
(def color *color/rocket-metallic* (138 129 141))
(def color *color/roman-silver* (131 137 150))
(def color *color/rose* (255 0 127))
(def color *color/rose-bonbon* (249 66 158))
(def color *color/rose-ebony* (103 72 70))
(def color *color/rose-gold* (183 110 121))
(def color *color/rose-madder* (227 38 54))
(def color *color/rose-pink* (255 102 204))
(def color *color/rose-quartz* (170 152 169))
(def color *color/rose-taupe* (144 93 93))
(def color *color/rose-vale* (171 78 82))
(def color *color/rosewood* (101 0 11))
(def color *color/rosso-corsa* (212 0 0))
(def color *color/rosy-brown* (188 143 143))
(def color *color/royal-azure* (0 56 168))
(def color *color/royal-blue-traditional* (0 35 102))
(def color *color/royal-blue-web* (65 105 225))
(def color *color/royal-fuchsia* (202 44 146))
(def color *color/royal-purple* (120 81 169))
(def color *color/royal-yellow* (250 218 94))
(def color *color/ruber* (206 70 118))
(def color *color/rubine-red* (209 0 86))
(def color *color/ruby* (224 17 95))
(def color *color/ruby-red* (155 17 30))
(def color *color/ruddy* (255 0 40))
(def color *color/ruddy-brown* (187 101 40))
(def color *color/ruddy-pink* (225 142 150))
(def color *color/rufous* (168 28 7))
(def color *color/russet* (128 70 27))
(def color *color/russian-green* (103 146 103))
(def color *color/russian-violet* (50 23 77))
(def color *color/rust* (183 65 14))
(def color *color/rusty-red* (218 44 67))
(def color *color/sacramento-state-green* (0 86 63))
(def color *color/saddle-brown* (139 69 19))
(def color *color/safety-yellow* (238 210 2))
(def color *color/saffron* (244 196 48))
(def color *color/st-patricks-blue* (35 41 122))
(def color *color/salmon* (255 140 105))
(def color *color/salmon-pink* (255 145 164))
(def color *color/sand* (194 178 128))
(def color *color/sand-dune* (150 113 23))
(def color *color/sandstorm* (236 213 64))
(def color *color/sandy-brown* (244 164 96))
(def color *color/sandy-taupe* (150 113 23))
(def color *color/sangria* (146 0 10))
(def color *color/sap-green* (80 125 42))
(def color *color/sapphire* (15 82 186))
(def color *color/sapphire-blue* (0 103 165))
(def color *color/satin-sheen-gold* (203 161 53))
(def color *color/scarlet* (255 36 0))
(def color *color/scarlet-crayola* (253 14 53))
(def color *color/schauss-pink* (255 145 175))
(def color *color/school-bus-yellow* (255 216 0))
(def color *color/screamin-green* (118 255 112))
(def color *color/sea-blue* (0 105 148))
(def color *color/sea-green* (46 255 139))
(def color *color/seal-brown* (50 20 20))
(def color *color/seashell* (255 245 238))
(def color *color/selective-yellow* (255 186 0))
(def color *color/sepia* (112 66 20))
(def color *color/shadow* (138 121 93))
(def color *color/shampoo* (255 207 241))
(def color *color/shamrock-green* (0 158 96))
(def color *color/sheen-green* (143 212 0))
(def color *color/shimmering-blush* (217 134 149))
(def color *color/shocking-pink* (252 15 192))
(def color *color/shocking-pink-crayola* (255 111 255))
(def color *color/sienna* (136 45 23))
(def color *color/html_silver* (192 192 192))
(def color *color/silver-chalice* (172 172 172))
(def color *color/silver-lake-blue* (93 137 186))
(def color *color/silver-pink* (190 174 173))
(def color *color/silver-sand* (191 193 194))
(def color *color/sinopia* (203 65 11))
(def color *color/skobeloff* (0 116 116))
(def color *color/sky-blue* (135 206 235))
(def color *color/sky-magenta* (207 113 175))
(def color *color/slate-blue* (106 90 205))
(def color *color/slate-gray* (112 128 144))
(def color *color/smitten* (200 65 134))
(def color *color/smoke* (115 130 118))
(def color *color/smokey-topaz* (147 61 65))
(def color *color/smoky-black* (16 12 8))
(def color *color/snow* (255 250 239))
(def color *color/soap* (206 200 118))
(def color *color/sonic-silver* (117 117 117))
(def color *color/space-cadet* (29 41 81))
(def color *color/spanish-bistre* (128 117 50))
(def color *color/spanish-carmine* (209 0 71))
(def color *color/spanish-crimson* (229 26 76))
(def color *color/spanish-orange* (232 97 0))
(def color *color/spanish-sky-blue* (0 255 255))
(def color *color/spiro-disco-ball* (15 192 252))
(def color *color/spring-bud* (167 252 0))
(def color *color/spring-green* (0 255 127))
(def color *color/star-command-blue* (0 123 184))
(def color *color/steel-blue* (70 130 180))
(def color *color/steel-pink* (204 51 204))
(def color *color/stil-de-grain-yellow* (250 218 94))
(def color *color/stizza* (153 0 0))
(def color *color/stormcloud* (79 102 106))
(def color *color/straw* (228 217 111))
(def color *color/strawberry* (252 90 141))
(def color *color/sunglow* (255 204 51))
(def color *color/sunset* (250 214 165))
(def color *color/super-pink* (207 107 169))
(def color *color/html_tan* (210 180 140))
(def color *color/tangelo* (249 77 0))
(def color *color/tangerine* (242 133 0))
(def color *color/tango-pink* (228 113 122))
(def color *color/taupe* (72 60 50))
(def color *color/taupe-gray* (139 133 137))
(def color *color/tea-green* (208 240 192))
(def color *color/tea-rose-orange* (248 131 121))
(def color *color/tea-rose-rose* (244 194 194))
(def color *color/html_teal* (0 128 128))
(def color *color/teal-blue* (54 117 136))
(def color *color/teal-deer* (153 230 179))
(def color *color/teal-green* (0 130 127))
(def color *color/telemagenta* (207 52 118))
(def color *color/tenné-tawny* (205 87 0))
(def color *color/terra-cotta* (226 114 91))
(def color *color/thistle* (216 191 216))
(def color *color/thulian-pink* (222 111 161))
(def color *color/tickle-me-pink* (252 137 172))
(def color *color/tiffany-blue* (10 186 181))
(def color *color/tigers-eye* (224 141 60))
(def color *color/timberwolf* (219 215 210))
(def color *color/titanium-yellow* (238 230 0))
(def color *color/tomato* (255 99 71))
(def color *color/toolbox* (116 108 192))
(def color *color/topaz* (255 200 124))
(def color *color/tractor-red* (253 14 53))
(def color *color/trolley-grey* (128 128 128))
(def color *color/tropical-rain-forest* (0 117 94))
(def color *color/true-blue* (0 115 207))
(def color *color/tufts-blue* (65 125 193))
(def color *color/tulip* (255 135 141))
(def color *color/tumbleweed* (222 170 136))
(def color *color/turkish-rose* (181 114 129))
(def color *color/html_turquoise* (48 213 200))
(def color *color/turquoise-blue* (0 255 239))
(def color *color/turquoise-green* (160 214 180))
(def color *color/tuscan* (250 214 165))
(def color *color/tuscan-brown* (111 78 55))
(def color *color/tuscan-red* (124 48 48))
(def color *color/tuscan-tan* (160 123 91))
(def color *color/tuscany* (192 153 153))
(def color *color/twilight-lavender* (138 73 107))
(def color *color/tyrian-purple* (102 2 60))
(def color *color/ua-blue* (0 51 170))
(def color *color/ua-red* (217 0 76))
(def color *color/ube* (136 120 195))
(def color *color/ucla-blue* (83 104 149))
(def color *color/ucla-gold* (255 179 0))
(def color *color/ufo-green* (60 208 112))
(def color *color/ultramarine* (18 10 143))
(def color *color/ultramarine-blue* (65 102 245))
(def color *color/ultra-pink* (255 111 255))
(def color *color/umber* (99 81 71))
(def color *color/unbleached-silk* (255 221 202))
(def color *color/united-nations-blue* (91 146 229))
(def color *color/university-of-california-gold* (183 135 39))
(def color *color/unmellow-yellow* (255 255 102))
(def color *color/up-forest-green* (1 68 33))
(def color *color/up-maroon* (123 17 19))
(def color *color/upsdell-red* (174 32 41))
(def color *color/urobilin* (225 173 33))
(def color *color/usafa-blue* (0 79 152))
(def color *color/usc-cardinal* (153 0 0))
(def color *color/usc-gold* (255 204 0))
(def color *color/university-of-tennessee-orange* (247 127 0))
(def color *color/utah-crimson* (211 0 63))
(def color *color/vanilla* (243 229 171))
(def color *color/vanilla-ice* (243 143 169))
(def color *color/vegas-gold* (197 179 88))
(def color *color/venetian-red* (200 8 21))
(def color *color/verdigris* (67 179 174))
(def color *color/vermilion-cinnabar* (227 66 52))
(def color *color/vermilion-plochere* (217 56 30))
(def color *color/veronica* (160 32 240))
(def color *color/violet* (143 0 255))
(def color *color/violet-color-wheel* (127 0 255))
(def color *color/violet-ryb* (134 1 175))
(def color *color/violet-web* (238 130 238))
(def color *color/violet-blue* (50 74 178))
(def color *color/violet-red* (247 83 148))
(def color *color/viridian* (64 130 109))
(def color *color/vivid-auburn* (146 39 36))
(def color *color/vivid-burgundy* (159 29 53))
(def color *color/vivid-cerise* (218 29 129))
(def color *color/vivid-orchid* (204 0 255))
(def color *color/vivid-sky-blue* (0 204 255))
(def color *color/vivid-tangerine* (255 160 137))
(def color *color/vivid-violet* (159 0 255))
(def color *color/warm-black* (0 66 66))
(def color *color/waterspout* (164 244 249))
(def color *color/wenge* (100 84 82))
(def color *color/wheat* (245 222 179))
(def color *color/white-smoke* (245 245 245))
(def color *color/wild-blue-yonder* (162 173 208))
(def color *color/wild-orchid* (215 112 162))
(def color *color/wild-strawberry* (255 67 164))
(def color *color/wild-watermelon* (252 108 133))
(def color *color/windsor-tan* (167 85 2))
(def color *color/wine* (114 47 55))
(def color *color/wine-dregs* (103 49 71))
(def color *color/wisteria* (201 160 220))
(def color *color/wood-brown* (193 154 107))
(def color *color/xanadu* (115 134 120))
(def color *color/yale-blue* (15 77 146))
(def color *color/yankees-blue* (28 40 65))
(def color *color/yellow-munsell* (239 204 0))
(def color *color/yellow-ncs* (255 211 0))
(def color *color/yellow-process* (255 239 0))
(def color *color/yellow-ryb* (254 254 51))
(def color *color/yellow-green* (154 205 50))
(def color *color/yellow-orange* (255 174 66))
(def color *color/yellow-rose* (255 240 0))
(def color *color/zaffre* (0 20 168))
(def color *color/zinnwaldite-brown* (44 22 8))
(def color *color/zomp* (57 167 142))

;;;;;;
;;; API

(def function color= (color-1 color-2)
  (and (= (alpha-of color-1) (alpha-of color-2))
       (= (red-of color-1) (red-of color-2))
       (= (green-of color-1) (green-of color-2))
       (= (blue-of color-1) (blue-of color-2))))

(def function color=* (color-1 color-2)
  (if (and color-1 color-2)
      (color= color-1 color-2)
      (eq color-1 color-2)))

(def function color/interpolate (color-1 color-2 ratio)
  (make-style/color (+ (* (- 1 ratio) (red-of color-1))
                       (* ratio (red-of color-2)))
                    (+ (* (- 1 ratio) (green-of color-1))
                       (* ratio (green-of color-2)))
                    (+ (* (- 1 ratio) (blue-of color-1))
                       (* ratio (blue-of color-2)))
                    (+ (* (- 1 ratio) (alpha-of color-1))
                       (* ratio (alpha-of color-2)))))

(def function color/lighten (color ratio)
  (color/interpolate color *color/white* ratio))

(def function color/darken (color ratio)
  (color/interpolate color *color/black* ratio))

(def function color/lighten/selection (color selection &optional (default-color color))
  (if selection
      (color/lighten color (/ (max 0 (min 6 (- (length selection) 4))) 6))
      default-color))

(def function color/darken/selection (color selection &optional (default-color color))
  (if selection
      (color/darken color (- 1 (/ (max 0 (min 6 (- (length selection) 4))) 6)))
      default-color))
